@@ -1,10 +1,10 @@
 /*
 **  This program checks to see if your version of setreuid works.
-**  Compile it, make it setuid root, and run it as yourself (NOT as
+**  Compile it, make it set-user-ID root, and run it as yourself (NOT as
 **  root).  If it won't compile or outputs any MAYDAY messages, don't
 **  define HASSETREUID in conf.h.
 **
-**  Compilation is trivial -- just "cc t_setreuid.c".  Make it setuid,
+**  Compilation is trivial -- just "cc t_setreuid.c".  Make it set-user-ID,
 **  root and then execute it as a non-root user.
 */
 
@@ -13,7 +13,7 @@
 #include <stdio.h>
 
 #ifndef lint
-static char id[] = "@(#)$Id: t_setreuid.c,v 8.4 1999/08/28 00:25:28 gshapiro Exp $";
+static char id[] = "@(#)$Id: t_setreuid.c,v 8.7 2001/09/08 01:21:13 gshapiro Exp $";
 #endif /* ! lint */
 
 #ifdef __hpux
@@ -23,10 +23,10 @@ static char id[] = "@(#)$Id: t_setreuid.c,v 8.4 1999/08/28 00:25:28 gshapiro Exp
 static void
 printuids(str, r, e)
 	char *str;
-	int r, e;
+	uid_t r, e;
 {
-	printf("%s (should be %d/%d): r/euid=%d/%d\n", str, r, e,
-		getuid(), geteuid());
+	printf("%s (should be %d/%d): r/euid=%d/%d\n", str, (int) r, (int) e,
+	       (int) getuid(), (int) geteuid());
 }
 
 int
@@ -41,7 +41,7 @@ main(argc, argv)
 
 	if (geteuid() != 0)
 	{
-		printf("SETUP ERROR: re-run setuid root\n");
+		printf("SETUP ERROR: re-run set-user-ID root\n");
 		exit(1);
 	}
 
@@ -75,7 +75,7 @@ main(argc, argv)
 	if (setreuid(realuid, 0) < 0)
 	{
 		fail++;
-		printf("setreuid(%d, 0) failure\n", realuid);
+		printf("setreuid(%d, 0) failure\n", (int) realuid);
 	}
 	printuids("after setreuid(realuid, 0)", realuid, 0);
 
@@ -115,7 +115,7 @@ main(argc, argv)
 	if (setreuid(realuid, 0) < 0)
 	{
 		fail++;
-		printf("setreuid(%d, 0) failure\n", realuid);
+		printf("setreuid(%d, 0) failure\n", (int) realuid);
 	}
 	printuids("after setreuid(realuid, 0)", realuid, 0);
 
